#ifndef __IMMS_H
#define __IMMS_H

#include <string>
#include <fstream>
#include <memory>
#include <map>

#include "immsdb.h"
#include "songinfo.h"

#define HOUR (60)
#define DAY (24*HOUR)

#define FIRST_SKIP_RATE -6
#define CONS_SKIP_RATE -3
#define FIRST_NON_SKIP_RATE 6
#define JUMPED_FROM -1
#define JUMPED_TO_SKIPPED 1
#define JUMPED_TO_FINNISHED 8
#define CONS_NON_SKIP_RATE 2

#define SAMPLE_SIZE 20
#define MIN_SAMPLE_SIZE 10
#define DISPERSION_FACTOR 3.8
#define MAX_TIME 20*DAY
#define MAX_RATING 150
#define MIN_RATING 60
#define CORRELATION_FACTOR 3

#define TERM_WIDTH 80

using std::string;
using std::ofstream;
using std::endl;
using std::auto_ptr;
using std::map;

typedef map<int, unsigned int> CandidateMap;

// IMMS, UMMS, we all MMS for XMMS?

class Imms
{
public:
    Imms();

    // Public interface
    int select_next();
    bool add_candidate(int playlist_num, string path, bool urgent = false);

    void start_song(string path);
    void end_song(bool at_the_end, bool jumped, bool bad);

    void playlist_changed();

    void setup(const char* _email, int _quality);

protected:
    // Helper functions
    bool fetch_song_info(const string &path);
    void parse_song_info(const string &path);
    void print_song_info(const string &path);
    int select_next_helper();

    // State variables
    string email;
    bool last_skipped, last_jumped;
    int quality, timehold, next_timehold, last_handpicked;
    int max_composite, min_composite, have_candidates;

    // Helper objects
    auto_ptr<ImmsDb> immsdb;
    ofstream fout;
    SongInfo songinfo;
    CandidateMap candidates;

    // Song info cache
    int rating, link; 
    bool identified, unrated;
    time_t last_played;
    string artist, title;
};

#endif
