#ifndef __IMMSDB_H
#define __IMMSDB_H

#include <string>
#include <utility>
#include <time.h>

#include "sqldb.h"

using std::string;
using std::pair;

typedef pair<string, string> StringPair;

#define SCHEMA_VERSION 2

#define CORRELATION_TIME 20*60  // 40 minutes

class ImmsDb : protected SqlDb
{
public:
    ImmsDb();
    ~ImmsDb();
    int identify(const string &path, time_t modtime);
    int identify(const string &path, time_t modtime, const string &checksum);

    int get_sid() { return sid; }
    int get_rating();
    int correlate(int from);
    time_t get_last();
    StringPair get_info();

    void set_last(time_t last);
    void set_title(const string &_title);
    void set_artist(const string &_artist);
    void set_rating(int rating);

    void add_recent(int weight);

    bool check_artist(string &artist);
    bool check_title(string &title);

protected:
    void register_new_sid(int new_sid = -1);
    void expire_recent(const string& where_clause);
    void sql_create_tables();
    void sql_schema_upgrade();

    // state cache
    int uid, sid;
    string artist, title;
};

#endif
