#include <iostream>
#include <sqlite.h>

#include "sqldb.h"
#include "strmanip.h"

using std::endl;
using std::cerr;

// Fuzzy compare function using levenshtein string distance
static void fuzzy_like(sqlite_func *context, int arg, const char **argv)
{
  if (argv[0] == 0 || argv[1] ==0)
      return;
  sqlite_set_result_int(context, string_like(argv[0], argv[1]));
}

SqlDb::SqlDb(const string &dbname) : nrow(0), ncol(0), resultp(0), errmsg(0)
{
    db = sqlite_open(dbname.c_str(), 600, &errmsg);
    if (!db)
        cerr << "Failed to open database '" << dbname << "'" << endl;
    else
        sqlite_create_function(db, "like", 2, fuzzy_like, 0);
}

SqlDb::~SqlDb()
{
    sqlite_free_table(resultp);
    close_database();
}

void SqlDb::close_database()
{
    if (db)
        sqlite_close(db);
    db = NULL;
}

void SqlDb::run_query(const string &query)
{
    if (db)
    {
        sqlite_exec(db, query.c_str(), 0, 0, &errmsg);
        handle_error(query);
    }
    else
    {
        cerr << "Database not open!" << endl;
    }
}

void SqlDb::select_query(const string &query)
{
    if (db)
    {
        sqlite_free_table(resultp);

        sqlite_get_table(db, query.c_str(), &resultp, &nrow, &ncol, &errmsg);
        handle_error(query);
    }
    else
    {
        cerr << "Database not open!" << endl;
    }
}

void SqlDb::handle_error(const string &query)
{
    if (errmsg
            && !strstr(errmsg, "already exists")
            && !strstr(errmsg, "uniqueness constraint failed")
            && !strstr(errmsg, "no such table"))
    {
        cerr << errmsg << endl;
        cerr << "while executing: " << query << endl;
    }
    free(errmsg);
    errmsg = 0;
}
